package vishnu.testvis.dendro;

/* Related word hierarchical clustering using a given triangle similarity matrix */

//import cluster.*;
import java.util.*;
import vishnu.testvis.object.*;

public class Clustering
{
    public static final int COMPLETE_LINKAGE = 0;
    public static final int AVERAGE_LINKAGE  = 1;
    public static final int SINGLE_LINKAGE   = 2;

    private Vector [] theResult;
    private double [][] theCentroids;
    private Cluster theRoot;
    public DataManager dataManager;
    private int method = 1;

    public Vector [] getTheResults()
    {
        return theResult;
    }
    public double[][] getTheCentroids()
    {
        return theCentroids;
    }
    
    public Cluster getRoot()
    {
        return theRoot;
    }


    public Clustering (DataManager dm, int linkage, float [][]m,
		       int num_of_vectors,
		       int arraySize, int min_c_items,
		       int target_clusters, int limit)
    {
	
        SimMatrix sim_matrix;
	LinkFunc linkage_func=null;
	dataManager = dm;
	
	// number of keywords
	int VECTOR_SIZE = arraySize;
        
	Cluster toplevel_cluster;
	TheInt clusters = new TheInt(0);
	
	Distance sim_func = new EuclideanDistance();
	
	double[] heights;
	double [][] vectors = floatToDoubleMatrix(m);//intToDoubleMatrix(m);
	//int [] coding = new int[m.length];
        
	// populate array with numbers from 0 to #docs-1
	//for (int c=0;c<m.length;c++) coding[c]=c;
	
	int i, j;
	
	switch (linkage)
	    {
    	    case COMPLETE_LINKAGE:
      		linkage_func = new MaxLink();
      		break;
    	    case AVERAGE_LINKAGE:
      		linkage_func = new AveLink();
      		break;
    	    case SINGLE_LINKAGE:
      		linkage_func = new MinLink();
      		break;
    	    default:
      		System.out.println("clustering.c: main(): linkage type "+
	                  linkage +" not defined.");
      	        System.exit(0);
    	}

        long startTime = System.currentTimeMillis();
        long readTime = System.currentTimeMillis();
        
        
        // if fewer documents than upper cluster limit
        // I think this is an error, should be vectors.length, viz. #documents
        // which was never spotted as limit is always smaller
        // than #keywords
        // if(limit > vectors[0].length) limit = vectors[0].length;

        // contains distance matrix between documents, built using
        // the double vector[i][] list of keyword frequencies


	Vector indices = null;
	//randomly choose sqrt(k*n) documents

	if( method == 0 )
	    {

		int sampleSize = 2*(int)Math.sqrt(vectors.length);
		Vector sampleInt = new Vector();
		Vector allDocs = new Vector();
		
		for( int s=0; s<vectors.length;s++ )
		    {
			allDocs.addElement(new Integer(s));
		    }
		
		while( sampleInt.size() < sampleSize )
		    {
			int r = (int)(Math.random()*allDocs.size());
			sampleInt.addElement(new Integer(r));
			allDocs.removeElementAt(r);
		    }
		double[][] sample_matrix = new double[sampleSize][VECTOR_SIZE];
		for(int s=0; s<sampleSize;s++)
		    {
			int docNum = ((Integer)sampleInt.elementAt(s)).intValue();
			sample_matrix[s] = vectors[docNum];
		    }
		
		indices = sampleInt;
		sim_matrix = new SimMatrix(sample_matrix,limit,sim_func,VECTOR_SIZE);

		long simMatixTime = System.currentTimeMillis();
		
		// an array of heights sorted by descending disim
		// as many heights as there are docs
		heights = new double[m.length];//[limit - 1];
		
		//int maxDoc = 0;
		int maxDoc = indices.size();
		
		Hierarchical hierarchical = new Hierarchical(maxDoc);
		theRoot = hierarchical.clustering(dm, sim_matrix, maxDoc, heights,linkage_func,indices);
		
		// collect all clusters
		Vector content=new Vector();
		getLeaves(theRoot,content);
		
		for( int it = 0; it < allDocs.size(); it++ )
		    {
			int docNum = ((Integer)allDocs.elementAt(it)).intValue();
			
			double min_dist = 10000;
			int min_index = -1;

			for( int c = 0; c < content.size(); c++ )
			    {
				
				Cluster clust = (Cluster)content.elementAt(c);

				int docNums = ((Integer)clust.getDocVect().elementAt(0)).intValue();

				double sim = sim_func.get(vectors[docNums],vectors[docNum], VECTOR_SIZE);

				if (sim < min_dist)
				    {
					min_dist = sim;
					min_index = c;
				    }
			    }
			
			Cluster clust = (Cluster)content.elementAt(min_index);
			clust._iter_items.addElement(new TheInt(docNum));
			clust._iter_items_num++;
			clust.docVect.addElement(new Integer(docNum));
			long iterativeTime = System.currentTimeMillis();
			
		    }
	    }			 
	else
	    {
   		Vector v = new Vector();
		for( int l = 0; l < vectors.length; l++ )
		    {
			v.addElement(new Integer(l));
		    }

		indices = v;
		sim_matrix = new SimMatrix(vectors, limit, sim_func, VECTOR_SIZE);
	        
		long simMatixTime = System.currentTimeMillis();
		
		// an array of heights sorted by descending disim
		// as many heights as there are docs
		heights = new double[m.length];//[limit - 1];
		
		//int maxDoc = 0;
		int maxDoc = indices.size();
		
		Hierarchical hierarchical = new Hierarchical(maxDoc);
		theRoot = hierarchical.clustering(dm, sim_matrix, maxDoc, heights,linkage_func,indices);
		
		// collect all clusters
		Vector content=new Vector();
		getLeaves(theRoot,content);
	    }
	
    }

    static double[][] intToDoubleMatrix(int [][] m)
    {
        int h=m.length;
        double [][] res = null;
        if (h > 0)
        {
          int w = m[0].length;
          res = new double[h][w];
          for(int c=0;c<h;c++)
            for(int d = 0; d<w; d++)
              res[c][d]=(double)m[c][d];
        }
        return res;
   }
    static double[][] floatToDoubleMatrix(float [][] m)
    {
        int h=m.length;
        double [][] res = null;
        if (h > 0)
        {
          int w = m[0].length;
          res = new double[h][w];
          for(int c=0;c<h;c++)
            for(int d = 0; d<w; d++)
              res[c][d]=(double)m[c][d];
        }
        return res;
   }


    private void getLeaves(Cluster node, Vector content)
    {
	if( node._child_1 != null )
	    {
		getLeaves(node._child_1,content);
		getLeaves(node._child_2,content);
	    }
	else
	    {
		content.addElement(node);
		Vector v = node.docVect;
		int i = ((Integer)v.elementAt(0)).intValue();
	    }
    }
    

}
